#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __PRINTING__
#include <Printing.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif


//-------------------------------------------------------------------------------------------

const unsigned short kCurrentLastBackCall = 0x0014;


extern pascal short GetLastBackCall (ProcPtr backPtr)
//	This back call returns the number of the last implemented back call. This can be used to
//	check for the existence of a specific back call in different versions of RagTime.
//	GetLastBackCall allows for the addition of back calls in the future without having to
//	change the interface version number. See the back call table above to determine the
//	number of a particular back call.
// NOTE: This back call is only available in RagTime 3.2 or newer.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0000, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void GetNextParam (ProcPtr backPtr)
//	This procedure gets the next parameter for external functions during calculation and
//	places it into the record pointed to by the parameter field of the RTXRecord. If the
//	parameter field is a NIL pointer after this call, then there are no more parameters.
//	Please refer to the section Passing Parameters and Results for more information.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0001, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void GetCell (long sheet, long document,
						   		 short col, short row,
						   		 RTValue *value, short typeMask, ProcPtr backPtr)
//	This procedure gets the contents of a particular cell. sheet is a reference number to a
//	particular spreadsheet, document is a reference number to a particular document, col and
//	row indicate the location within the spreadsheet, value is an RTValue record to which
//	the result is returned, and typeMask indicates to RagTime which data types the caller is
//	interested in. If a text value is received in the value parameter, then it is the
//	external function groups responsibility to dispose of the block associated with the
//	handle eTextHdl.
//	The typeMask parameter can contain one or more of the value type masks defined at the
//	beginning of the interface. All values are converted to a value which is allowed in the
//	typeMask. If conversion is not possible, the error value valErr is returned, unless
//	valMaskErr is not in the mask. In this case, GetCell does not return, and the error
//	VALUE! is returned by the function. If the cell is empty, the value valNil is
//	returned, unless valMaskNil is not in the mask, in which case RagTime returns a default
//	value for the cell. The default values are zero for valMaskIntNum and valMaskNum, an
//	empty string for valMaskStr, and false for valMaskBool. If the cell referred to
//	contains an error value, then RagTime does not return, no matter what is defined in the
//	mask, and the result returned in the spreadsheet for your function is the same error
// value as that of the cell in question.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0002, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal Boolean GetNextCell (long sheet, long document,
						   	      	  short *col, short *row,
									 		  const Rect *range, RTValue *value,
									 		  short typeMask, ProcPtr backPtr)
//	This function searches the range of sheet in document, starting at col, row, and sets
//	col, row to the first non-empty cell found that contains a value allowed by typeMask.
//	The function searches one row at a time, checking each column in the row. Any values
//	outside the range are automatically adjusted, so that a search can be continued simply
//	by adding 1 to the col parameter. If range is NIL, then the whole sheet is searched.
//	valMaskErr is used in the same way as in GetCell. Unlike GetCell, however, GetNextCell
//	ignores valNil if it is in the typeMask.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0003, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void SetCell (long sheet, long document,
									 short col, short row,
						   		 RTValue *value, ProcPtr backPtr)
//	This procedure puts value into cell col, row of the spreadsheet sheet in document
//	document. Even though value is a VAR parameter, it is not changed by this back call, and
//	if value contains a text value, then the external function group is responsible for
//	disposing of the handle.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0004, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void FindSheet (short pageNum, short frameNum,
							  		   StringPtr path, Boolean openDocu,
							 		   long *sheet, long *document,
							 		   ProcPtr backPtr)		
//	This procedure searches for the document with name path^. If path is NIL, the current
//	document is used. If path^ is just a file name without any colons, only documents which
//	are open and visible to the user are returned. If path^ contains any colons, it is taken
//	as a full or partial path name. The partial path name starts at the location of the
//	current document (or the program, if the document is not saved yet). In this case, all
//	open documents, whether visible to the user or not, are always returned. Closed
//	documents are opened if openDocu is TRUE. If a document is found, the spreadsheet in the
//	frame with the number frameNum on page pageNum is searched. If found, it is returned in
//	sheet, otherwise, sheet is set to zero.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0005, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void CalcAFunction (short groupStart, short functionIndex,
											 long document, long sheet,
											 ProcPtr backPtr)
//	This call is used to trigger recalculation of individual functions from an external
//	function group. In order for this back call to work correctly, you must first set up the
//	function that you want recalculated so that its needsRef flag is set in the 'BXDe'
//	resource. Recalculation of the function is started by passing the groupStart field of
//	the RTXRecord, and the number of the function that needs to be recalculated in document,
//	sheet.document and/or sheet may be zero. When document is zero, all occurrences of the
//	function in question in all sheets in all documents are recalculated. When document
//	contains a valid number and sheet is zero, all occurrences of the function in question
//	in all sheets of the document are recalculated.
//	This routine only sets a flag in RagTime that calculation must be carried out.
//	Calculation does not actually begin until after the external function returns.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0006, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal Boolean Text2Value (Ptr textPtr, short textLen,
								   		 RTValue *result, ProcPtr backPtr)
//	This is the RagTime text parser. It tries to convert the given text to a number or a
//	date. The rules are exactly the same as during manual data entry. If a conversion is not
//	possible, Text2Value returns FALSE and result remains unchanged.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0007, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void PrivateToDeskScrap (ProcPtr backPtr)
//	Before reading the desk scrap, an external function group should call this procedure to
//	make sure that RagTime converts its private scrap to the desk scrap.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0008, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void CalcNow (ProcPtr backPtr)
//	Triggers recalculation of everything. If calculation is set to automatic, calculation
//	starts after the call; otherwise, it starts as soon as the user selects the command
//	Calculate All. This back call may only be used if the function group is called with one
//	of the special purpose selectors. This function was added for HandiWorks/RagTime Classic
//	2, and is here for compatibility only. It triggers a recalculation of ALL formulas in
//	the document (in the same way as option-shift-command Calculate All). This interface
//	provides the function CalcAFunction, which provides a more specific means of controlling
//	recalculation than CalcNow. As opposed to HandiWorks/RagTime Classic 2, which does the
//	calculation synchronously, CalcNow only triggers recalculation in RagTime 3.1 or 3.1|7
//	and returns before calculation actually starts.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0009, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal short MailMerge (Boolean printStop, ProcPtr backPtr)
//	This function returns the number of the current print in the print cycle. It stops
//	printing if the stopPrinting argument is set to TRUE, and always returns 1 if printing
//	is not in progress. Currently there is no clean way to tell whether printing is in
//	progress, and you shouldnt need to know. Dont forget to set the kIsMailMerge and
//	kNeedsReference flags for every function using MailMerge and the kHasMailMerge group
//	flag if you use MailMerge at all. RagTime uses these flags to determine whether it
//	should show the mail merge dialog before printing.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x000A, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal OSErr SaveDocument (long document, FSSpecPtr dest,
											 Boolean askForName, FSSpecPtr actualFile,
											 ProcPtr backPtr)
//	This function saves document to disk, returning any error as the function result. If the
//	dest argument is NIL, then this back call works just like the save commands from the
//	File menu. If the argument askForName is set to FALSE, this is treated just like a
//	selection of the Save menu item, and if askForName is set to TRUE, SaveDocument
//	performs the same action as a selection of the Save as menu item. A function return
//	value of 1 indicates that the user cancelled the operation. If the argument dest is a
//	pointer to an FSSpec record, then document is saved to that file, overwriting the file
//	without warning if it already exists (regardless of what type of file it is). The
//	argument askForName is ignored in this case. When this type of document save is
//	performed, the documents dirty flag is not changed.
//	Regardless of what type of save is done, successful completion sets actualFile to the
//	file that was used. If dest is specified, then the FSSpec pointed to by actualFile
//	always has the same values as the FSSpec pointed to by dest.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x000B, 0x4E90);

//-------------------------------------------------------------------------------------------

enum	{
	kTempPrintRecord 			 = 0x0001,
	kOnlyOneSheet 				 = 0x0002,
	kDoJobDialog 				 = 0x0004,
	kShowJobDialogExtensions = 0x0008,
	kShowProgress 				 = 0x0010
};

struct Page2Paper {
	short   p2pSize;
	Rect    printable;		//	printable area in document coordinates
									// Used to draw the dotted line on the screen
	Fixed   tlVOffset;		// offset to the top left corner of the
	Fixed   tlHOffset;		// top left sheet from top left corner of the page
	Point   sheets;			//	# of sheets
	short   flags;				//	see constants above
	Fixed   overLap;			//	amount of overlap
   long    spare1;
};
typedef	struct	Page2Paper	Page2Paper;

		
struct	FPrintRec {
	TPrint		macPrint;
	Page2Paper	p2p;
};
typedef struct FPrintRec FPrintRec, *FPrintPtr, **FPrintHdl;


enum	{
	kP2PCutMarks 			= 0,				//	draw registration and cut marks
	kP2PMaskCutMarks 		= 0x0001,

	kP2PRevertPages		= 1,				//	wether or not revert the page order
	kP2PMaskRevPages 		= 0x0002,

	kP2PScalingTrick		= 2,				//	use the scaling trick
	kP2PMaskScalTrick 	= 0x0004,

	kP2PColorPS				= 3,				//	the printer has color PostScript
	kP2PMaskColorPS 		= 0x0008,

	kP2PTransverse			= 4,				//	transverse the paper with 'setpageparams'
	kP2PMaskTrans 			= 0x0010,
	
	kP2PNoPSPat				= 5,				//	don't use PostScript patterns
	kP2PMaskNoPSPat 		= 0x0020,

	kP2PAscii				= 6,				//	Whether or not to download bitmaps as 7-bit ASCII.
	kP2PMaskAscii 			= 0x0040
};	

enum	{
	kPrBCMessageBeforeDialogs = 1,
	kPrBCMessageAfterDialogs,
	kPrBCMessageAfterOpenDoc,				// can be called more than once!
	kPrBCMessageAfterOpenPage,
	kPrBCMessageBeforeClosePage,
	kPrBCMessageBeforeCloseDoc,			// can be called more than once!
	kPrBCMessageBeforeSpooling,			// can be called more than once!
	kPrBCMessageAtEnd
};

struct PrintBackCallRecord {
	FPrintHdl   hPrint;
	FPrintHdl   basedOn;
	Point       pageSize;
	short		   copies;
	short		   currentCopy;
	short		   pages;
	short		   currentPage;
	short		   currentPageNo; // Note: total number of sheets is in hPrint^^.p2p.sheets
	Point			currentSheet;
};

typedef struct PrintBackCallRecord PrintBackCallRecord, *PrintBackCallRecordPtr;

//-------------------------------------------------------------------------------------------

extern pascal OSErr PrintDocument (long document, long flags,
									 		  THPrint hPrint, ProcPtr backCallFunction,
									 		  long backCallRefCon, ProcPtr backPtr)
// Print the given document to the current printer driver
//
//	pascal Boolean PrintBackCall (short message,
//											PrintBackCallRecordPtr data,
//											short externalResRef,
//											long backCallRefCon);
//
// 1: cancel
// iPrABort
//
// See the RagTime Add On Documentation for more information.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x000C, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void GetDocumentName (long document, StringPtr docuName,
									  			Boolean fullPath, ProcPtr backPtr)
//	Returns the name of document in the Pascal string pointed to by docuName. The Pascal
//	string pointed to by docuName must be provided by the caller. If fullPath is TRUE, then
//	the full path to the document is returned, in which case the string pointed to by
//	docuName must have a size of at least 256 bytes (that is, type Str255). If fullPath is
//	FALSE, then only the name of the document is returned, in which case the string pointed
//	to by docuName must have a size of at least 64 bytes (that is, type Str63).
 FOURWORDINLINE(0x205F, 0x3F3C, 0x000D, 0x4E90);

//-------------------------------------------------------------------------------------------

//	The next two calls allow creation, lookup and disposing of "data items". Data items are
//	saved with the document and can be retrieved later. At runtime, data items are just
//	Handles and can be used like Handles. For retrieving, a data item is identified by a
//	four character type and a 16 bit id, just like resources.
//
// PLEASE READ RAGTIME ADD ON DOCUMENTATION FOR MORE INFORMATION.

const short kDontCreateItem = -1;

//-------------------------------------------------------------------------------------------

extern pascal Handle GetDataItem (long document, OSType theType,
											 short theID, long size,
											 ProcPtr backPtr)
//	Lookup data item with type 'theType' and id 'theID' in 'document' and return its Handle.
//	If the data item is found, 'size' is ignored. Else the data item is created with this
//	size, if 'size' is not 'kDontCreateItem'. The contents of the new data item is cleared
//	to all 0. GetDataItem returns NIL if the item did not exist and 'size' is
//	'kDontCreateItem' or if creation wasnt possible for any reason, most likely memory
//	overflow. In the later case you should stop processing and return 'memFullErr' to
//	RagTime
 FOURWORDINLINE(0x205F, 0x3F3C, 0x000E, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void DeleteDataItem (long document, OSType theType,
									 		  short theID, ProcPtr backPtr)
//	Removes the given data item from the document and disposes its Handle. If there is no
//	such data item in the document, nothing happens
 FOURWORDINLINE(0x205F, 0x3F3C, 0x000F, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void DocumentChanged (long document, ProcPtr backPtr)
//	This routine marks the document document as being changed. This is the same as setting
//	the dirty flag, and it enables the Save menu item in the File menu of RagTime. Call
//	this after changing a data item. Creation of a data item automatically marks the
//	document as changed.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0010, 0x4E90);
 
//-------------------------------------------------------------------------------------------

extern pascal OSErr OpenPreferences (short *resRefNum, ProcPtr backPtr)
//	This function creates the RagTime preferences file if necessary and opens the resource
//	fork. An operating system error is returned if there are problems performing this
//	operation. If noErr is returned from the call to OpenPreferences, then the external
//	function is responsible for closing the preferences file with a call to CloseResFile
//	(resRefNum) before returning to RagTime. Please notice that opening the preferences file
//	changes the current chain of open resource files.
// (See the section Tips & Caveats below for more information about the chain of open
// resource files.)
//	To avoid problems with this situation, you should save the current resource file, call
//	OpenPreference, perform whatever actions need to be performed while you have access to
//	the preferences file, close the preferences file, then set the current resource back to
//	the one that you saved before calling OpenPreferences.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0011, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal void UpdateWindow (WindowPtr whichWindow, ProcPtr backPtr)
//	Update the given window, if its one of RagTimes windows. Else nothing happens. Usefull
//	if you set up a movable window in front of a RagTime window
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0012, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal Boolean GetCellAlways (long sheet, long document,
						   					 short col, short row,
						   			       RTValue *value, short typeMask, ProcPtr backPtr)
//	This function is identical to GetCell, with the following exceptions. If valMaskErr is
//	included in typeMask, then both conversion exceptions and cells which contain errors
//	will return value of type valErr. This function will return FALSE in case of a
//	conversion error, which allows callers to differentiate between conversion errors and
//	cells which have an error value.
//	NOTE: GetCellAlways ALWAYS returns IF valErr is in typeMask. If the cell is an error, or
//	there is a conversion error, and valErr is not in typeMask, this call will not return.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0013, 0x4E90);

//-------------------------------------------------------------------------------------------

extern pascal Boolean GetNextCellAlways (long sheet, long document,
									   	    	  short *col, short *row,
												 	  const Rect *range, RTValue *value,
												 	  short typeMask, Boolean* conversionWasSuccessful,
												 	  ProcPtr backPtr)
//	This function is identical to GetNextCell, with the following exceptions. If valMaskErr
//	is included in typeMask, then both conversion exceptions and cells which contain errors
//	will return value of type valErr. This function will set conversionWasSuccessful to
//	FALSE in case of a conversion error, which allows callers to differentiate between
//	conversion errors and cells which have an error value.
//	NOTE: GetNextCellAlways ALWAYS returns IF valErr is in typeMask. If the cell is an error,
//	or there is a conversion error, and valErr is not in typeMask, this call will not return.
 FOURWORDINLINE(0x205F, 0x3F3C, 0x0014, 0x4E90);

