#ifndef __MEMORY__
#include <Memory.h>
#endif


// The resource types used by the external function interface
#define	kRTXFuDescripRes	'BXDe'	// B&E eXternal functions Descriptor
#define	kRTXFuCodeRes		'BXCo'	// B&E eXternal functions Code
#define	kRTXFuResListRes	'BXRl'	// B&E eXternal functions resource maping list


#define  kRTXFuVersion		2
#define	kRTXFuVersion317	4
#define	kRTXFuVersion32	5
#define  kCurrentVersion	5

// Selector enumeration
enum	{
	// special purpose selectors
	kInitSelector 			=  0,				// Selector used for the initilization call
	kQuitSelector 			= -1,				// Selector used for the quit call

	// the following four messages are only sent if 'kWantsMessagesFlag' is set
	kOpenDocSelector 		= 	-2,			// a document was just opened
	kCloseDocSelector		= 	-3,			// a document is about to be closed
	kActivateSelector 	= 	-4,			// a document is activated
	kDeactivateSelector 	= 	-5,			// a document is deactivated
	kTimeSelector 			= 	-6				// Selector used for periodic calls (See 'kNeedsTimeFlag')
};


// The value types used by RagTime}
enum	{
	valNil					= 	0,				//	no value
	valNum 					=	1,				//	numerical value
	valBool					= 	2,				//	boolean value
	valDate					=	3,				//	date
	valText					=	4,				//	text
	valErr 					=	5,				//	error value
	valRTF					=	10,			//	Rich Text Format	RT 3.2 or newer only
	valIntNum				=	17,			// whole number of Pascal type LongInt
	valAll					=	32,
	cellRange				=	1000,			// used in RTValue to pass range parameters

	valMaskNil				=	0x01,
	valMaskNum				=	0x02,
	valMaskBool				=	0x04,  
	valMaskDate				=	0x08,  
	valMaskText				=	0x10,  
	valMaskErr				=	0x20,  
	valMaskIntNum			=	0x80,
	valMaskAll				=	0x9F
};



//	A function parameter is described by one integer:
//		bit 15:			0: optional parameter, 1: requested parameter
//		bit 13 and 14:	0:	no parameter
//						1:	single value requested
//						2:	list requested
//						3:	range requested
//		bit 0..12:		possible value kinds according to the bit numbers defined above and
//							the constants defined in RTXFuTypes.r


//-------------------------------------------------------------------------------------------
// RTCell
//-------------------------------------------------------------------------------------------

//	Coordinates of a spreadsheet cell. It's the same as a point, just other names
struct RTCell {
	short r;			// the row, 1 is row 1 in the range
	short	c;			// the column, 1 is column 1
};
typedef struct RTCell RTCell;

 

//-------------------------------------------------------------------------------------------
// RTParamRecord
//-------------------------------------------------------------------------------------------

//	maximum numbers of parameters - 1 for a function
#define	kRTMaxParas		6

//	Description of the parameters of a single function
struct RTParamRecord {
	unsigned	char		fuSpare;						// reserved for future use
	unsigned	char		fuFlags;						// bit 0: kNeedsReference (see below)
															// bit 1: kIsMailMerge (see below)
															//	all other bits are 0
	short					fuParas[kRTMaxParas];
};
typedef struct RTParamRecord	RTParamRecord;

//-------------------------------------------------------------------------------------------
// RTXFuDescript
//-------------------------------------------------------------------------------------------

//	Description of a single function

typedef unsigned char RTXFuName[80];
typedef Str31		    RTXFuInternalName;
struct  RTXFuDescript {
	RTParamRecord		paramRec;
	RTXFuName			fuUserName;
	RTXFuInternalName	fuInternalName;
};
typedef struct RTXFuDescript	RTXFuDescript;

//-------------------------------------------------------------------------------------------
// BXDe resource (C declaration)
//-------------------------------------------------------------------------------------------

struct	RTXFuList	{
	short					intfVersion;	//	version of the interface (current = kRTXFuVersion32)
	struct RTXFuList	**nextList;
	Handle				codeHdl;
	long					refCon;
	short					groupFlags;		//	reserved for future use, should be zero
	RTXFuName			groupText;		//	description of the function group
	short					groupVersion;	//	version of the function group, in percent (2.1 is 210)
	short					groupLanguage;	//	the localization language of this group (see ScriptManager)
	long					groupTime;		//	if kNeedsTimeFlag is set, the group is called every
												// 'groupTime' ticks. The time between two calls may actually
												// be longer due to system load
	short					resId;			//	resource id of the 'kRTXFuResListRes' for this function group
												//	Only valid in memory
	short					groupStart;		//	for the 'CalcAFunction' back call
	long					spare;
	short					numFuncts;
	RTXFuDescript		functs[100];
};
typedef struct RTXFuList RTXFuList, *RTXFuListPtr, **RTXFuListHdl;


//	names for 'groupFlags'
enum	{
	kNeedsTimeFlag     = 0,     // if set, the function group will be called with selector
										 // 'kTimeSelector' every 'groupTime' ticks
	kWantsMessagesFlag = 1,		 // if set, the kOpenDocSelector .. kDeactivateSelector
										 // messages are send to the group.
	kHasMailMerge      = 2		 // This group contains one or more functions which use the mail merge
										 // interface. If any of these functions (which must have the kIsMailMerge
										 // flag set in its paramRec.fuFlags) is used in a document, the document
										 // will do mail merge printing
};

//	names for flags in paramRec.fuFlags
enum	{
  kNeedsReference 	= 0,		 // every use of function is added to special list, allows
										 // use of CalcAFunction
  kIsMailMerge 		= 1		 // set this for functions which use the mail merge back call.
										 // If this is set, 'kNeedsReference' must be set, too
};
 

//-------------------------------------------------------------------------------------------
// BXRl resource (C declaration)
//-------------------------------------------------------------------------------------------
// Definition for tableFlags in RTXFuResTable. If set, the resource may not be renumbered by
// the XFuInstaller
#define  kResNoRenumberFlag   0 		


struct	RTXFuResTable {
	short    tableFlags;			//	flags, see definition
	ResType  resourceType;		// 'DITL' or something else
	short    externalID;		   // the external resource ID
};
typedef struct RTXFuResTable RTXFuResTable;


// definitions for resourceFlags in RTXFuResRec
const short kHasParameters = 0;	// IF set, there is a STR# resource with the same ID as the
											// function itself, containing the parameters for the functions
											// you'll porbably never need this one in your code

struct RTXFuResRec {
	short				numResources;	// This is 1-based
	short				resourceFlags;
	RTXFuResTable	resTable[1];	// cant use unsized arrays (i.e. resTable[]) in C++
											// actually there numResources elements in the array
};
typedef struct RTXFuResRec RTXFuResRec, *RTXFuResPtr, **RTXFuResHdl;


//	values must not be longer than kRTMaxValueLen.
//	(Actually only text values are of variable size) 
const unsigned short kRTMaxValueLen = 32000;


//	Error values are never passed as parameters but can be returned as function result.
//	If a parameter evaluation results in an error, evaluation is canceled and the
//	function is never called or GetNextParam does not return.
//	GetCell && GetNextCell however may return a value of type valErr 
enum	RTErrVal	{
	errNoErr 	= 0,		// no error
	errDivZ,					// division by zero in the SANE meaning
	errNum,					// numerical error other then 'errDivZ'
	errNA,					// not available
	errRef,					// reference error
	errVal,					// wrong type of value
	errDate,					// wrong date
	errRange,				// wrong index
	errComplex,				// calculation stack overflow
	errUnimpl				// unimplemented function
};
typedef enum RTErrVal RTErrVal;

// We need a 80 bit type. Note that for 68K code with the 68881 option switched OFF
// a long double has 80 bit. When the option is ON we have 96 bit and for PowerPC 64 bit!
typedef long double TExtended80;

//	A variant record is used for passing parameters and results
//	in C we use a union to accomplish the same thing and to be as close
//	as possible to the documentation 
struct RTValue {
	short valKind;
	union	{											
			TExtended80	eNumber;		// valNum, valDate (dates are stored as days since 1/1/1904)
			long	   	eIntNum;		// valIntNum
			Boolean		eBool;		//	valBool
			struct {						//	valText:
				short   	eTextLen;	// length of the text
				Handle  	eTextHdl;	// the text is in this handle. If you pass me a value, you
			} t;							// have to allocate and deallocate the handle. The exception
											// is the result (of course). If I pass you a parameter, I
											// will deallocate (or reuse) the handle the next time you
											// call GetNextParam. To avoid this, copy	eTextHdl to a local
											// variable and set it to nil. If you call GetCell or GetNextCell,
											// you have to deallocate the handle
			RTErrVal		eErr;			// valErr
			struct {						// cellRange:
				long		eDocument;	// the document which contains the range
				long		eSheet;		// the spreadsheet which contains the range
				Rect		eRange;		// the range
			} r;
	};
};
typedef struct RTValue RTValue, *RTValuePtr;


// Parameters are passed to the external functions in the following parameter block. This
// way of passing parameters provides for easy extensibility.
struct RTXRecord {
	long				refCon;			// <-> Used by the external function. Set to zero before the
											// first call. Preserved by RagTime between calls.
											// and will be saved for the next call.
	short				selector;		// --> which function is called
	RTValuePtr		parameter;		// --> points to the actual parameter
	RTValuePtr		resultDest;		// <-- Function result. RagTime creates the pointer, but the 
											// external function must fill the record pointed to by the pointer.
	ProcPtr			backPtr;			// --> RagTime entry point for back calls
	long				document;		// --> the current document for this call
											// If NIL, the call is document independent (like kInitSelector)
	long				sheet;			// --> the current sheet for this call.
											// If NIL, the call is sheet independent (like kOpenDocSelector)
	RTCell			cell;				// --> the cell of the formula. 0,0 if the call is cell independent
	short				resId;			// --> id of the 'BXDe' resource for this function group
	short				groupStart;		// --> for the 'CalcAFunction' back call
};
typedef struct RTXRecord RTXRecord, *RTXPtr;



// Prototype for main
pascal short main (RTXPtr callBlock);
